'use strict';
var yeoman = require('yeoman-generator');
var chalk = require('chalk');
var yosay = require('yosay');

module.exports = yeoman.generators.Base.extend({
  prompting: function () {
    var done = this.async();

    // Have Yeoman greet the user.
    this.log(yosay(
      'Welcome to the ' + chalk.red('Yulp') + ' generator!'
    ));

    var prompts = [
      {
        type: 'confirm',
        name: 'someOption',
        message: 'Would you like to enable this option?',
        default: true
      },
      {
        name: 'yourname',
        message: 'What is your name?',
        default: 'no name'
      }
    ];

    this.prompt(prompts, function (props) {
      this.props = props;
      // To access props later use this.props.someOption;

      done();
    }.bind(this));
  },

  writing: {
    app: function () {
      this.fs.copy( this.templatePath('app/_data/'), this.destinationPath('app/_data/') );
      this.fs.copy( this.templatePath('app/img/'), this.destinationPath('app/img/') );
      this.fs.copy( this.templatePath('app/js/'), this.destinationPath('app/js/') );
      this.fs.copy( this.templatePath('app/_stylus/'), this.destinationPath('app/_stylus/') );

      this.fs.copy( this.templatePath('app/*.ect'), this.destinationPath('app/') );
    },

    projectfiles: function () {
      this.fs.copy( this.templatePath('_bower.json'), this.destinationPath('bower.json') );
      this.fs.copy( this.templatePath('_bowerrc'), this.destinationPath('.bowerrc') );
      this.fs.copy( this.templatePath('_csslintrc'), this.destinationPath('.csslintrc') );
      this.fs.copy( this.templatePath('_editorconfig'), this.destinationPath('.editorconfig') );
      this.fs.copy( this.templatePath('_htmlhintrc'), this.destinationPath('.htmlhintrc') );
      this.fs.copy( this.templatePath('_iconTemplate.html'), this.destinationPath('_iconTemplate.html') );
      this.fs.copy( this.templatePath('_iconTemplate.styl'), this.destinationPath('_iconTemplate.styl') );
      this.fs.copy( this.templatePath('_jshintrc'), this.destinationPath('.jshintrc') );
      this.fs.copy( this.templatePath('_package.json'), this.destinationPath('package.json') );
      this.fs.copy( this.templatePath('_README.md'), this.destinationPath('README.md') );
      this.fs.copy( this.templatePath('_sprite-template.scss'), this.destinationPath('_sprite-template.scss') );
    },

    gitfiles: function () {
      this.fs.copy( this.templatePath('_gitignore'), this.destinationPath('.gitignore') );
    },

    gulpfiles: function () {
      this.fs.copy( this.templatePath('config.js'), this.destinationPath('config.js') );
      this.fs.copy( this.templatePath('gulpfile.js'), this.destinationPath('gulpfile.js') );

      this.fs.copy( this.templatePath('gulp/assetpaths.js'), this.destinationPath('gulp/assetpaths.js') );
      this.fs.copy( this.templatePath('gulp/autoprefixer.js'), this.destinationPath('gulp/autoprefixer.js') );
      this.fs.copy( this.templatePath('gulp/browserSync.js'), this.destinationPath('gulp/browserSync.js') );
      this.fs.copy( this.templatePath('gulp/clean.js'), this.destinationPath('gulp/clean.js') );
      this.fs.copy( this.templatePath('gulp/copy.js'), this.destinationPath('gulp/copy.js') );
      this.fs.copy( this.templatePath('gulp/csslint.js'), this.destinationPath('gulp/csslint.js') );
      this.fs.copy( this.templatePath('gulp/ect.js'), this.destinationPath('gulp/ect.js') );
      this.fs.copy( this.templatePath('gulp/htmlhint.js'), this.destinationPath('gulp/htmlhint.js') );
      this.fs.copy( this.templatePath('gulp/iconFont.js'), this.destinationPath('gulp/iconFont.js') );
      this.fs.copy( this.templatePath('gulp/imagemin.js'), this.destinationPath('gulp/imagemin.js') );
      this.fs.copy( this.templatePath('gulp/inject.js'), this.destinationPath('gulp/inject.js') );
      this.fs.copy( this.templatePath('gulp/jshint.js'), this.destinationPath('gulp/jshint.js') );
      this.fs.copy( this.templatePath('gulp/sprite.js'), this.destinationPath('gulp/sprite.js') );
      this.fs.copy( this.templatePath('gulp/stylus.js'), this.destinationPath('gulp/stylus.js') );
      this.fs.copy( this.templatePath('gulp/svgSprite.js'), this.destinationPath('gulp/svgSprite.js') );
      this.fs.copy( this.templatePath('gulp/useref.js'), this.destinationPath('gulp/useref.js') );
      this.fs.copy( this.templatePath('gulp/watch.js'), this.destinationPath('gulp/watch.js') );
      this.fs.copy( this.templatePath('gulp/wiredep.js'), this.destinationPath('gulp/wiredep.js') );
    }
  },

  install: function () {
    this.installDependencies();
  }
});
